package require snit
package provide ts::client 0.1

namespace eval ts {
    snit::type client {
        variable connection {}
        variable active_tuplespace {}

        destructor {
            catch {close $connection}
        }

        method Connected {} {
            expr {$connection ne {}}
        }

        method Connected_or_error {} {
            if {![$self Connected]} {
                return -level 2 -code error "not connected"
            }
        } 

        method Read_response {} {
            gets $connection resp
            while {![info complete $resp]} {
                append resp \n[gets $connection]
            }
            return $resp
        }

        method connect {{server localhost} {port 4040}} {
            if {$connection ne {}} {
                return -code error "already connected close current connection first"
            }
            set connection [socket $server $port]
            fconfigure $connection -buffering line
            return     
        }

        method disconnect {} {
            $self Connected_or_error
            close $connection
            set connection {}
        }

        method send {cmd args} {
            $self Connected_or_error
            puts $connection [list $cmd {*}$args]
            return [$self Read_response] 
        }

        method open {tuplespace} {
            $self Connected_or_error
            set active_tuplespace $tuplespace
            $self send open $tuplespace
        }

        method spaces {} {
            $self Connected_or_error
            # list the available tuplespaces on this server
            $self send dir

        }

        method dir {args} {
            $self Connected_or_error
            if {[llength $args]==1} {
                $self send dir {*}$args
            } elseif {[llength $args]==0} {
                $self send dir $active_tuplespace
            } else {
                return -code error "wrong # of args: should be \"$self dir ?tuplespace?\""
            }
        }

        method out {args} {
            $self Connected_or_error
            if {[llength $args]==2} {
                $self send out {*}$args
            } elseif {[llength $args]==1} {
                $self send out $active_tuplespace {*}$args
            } else {
                return -code error "wrong # of args: should be \"$self out ?tuplespace? tuple\""
            }
        }

        method in {args} {
            $self Connected_or_error
            if {[llength $args]==2} {
                $self send in {*}$args
            } elseif {[llength $args]==1} {
                $self send in $active_tuplespace {*}$args
            } else {
                return -code error "wrong # of args: should be \"$self in ?tuplespace? tuple\""
            }
        }

        method read {args} {
            $self Connected_or_error
            if {[llength $args]==2} {
                $self send read {*}$args
            } elseif {[llength $args]==1} {
                $self send read $active_tuplespace {*}$args
            } else {
                return -code error "wrong # of args: should be \"$self read ?tuplespace? tuple\""
            }
        }
    }
}

